package AppLib::String;

# The AppLib::String package contains generic string functions.
# Handy things that don't specificly belong in any one place

use strict;
use UNIVERSAL 'isa';
use base 'AppLib::Error';
use Class::Autouse qw{MIME::Base64};

# Set the buffer size.
# The MIME documents state that the buffer size should be a multiple
# of 57 bytes, so that it will come out at 76 bytes per line.
use vars qw{$base64_encode_buffer_size $base64_decode_buffer_size};
BEGIN {
	$base64_encode_buffer_size = 60 * 57;
	$base64_decode_buffer_size = 60 * 76;
}

sub humanFileSize {
	my $class = shift;
	my $size = shift || 0;

	if ( $size < 1024 ) {
		return $size . "B";

	} elsif ( $size < (1024 ** 2) ) {
		$size = $size / 1024;
		$size = $size < 10 ? sprintf( "%.1f", $size ) : int( $size );
		return $size . 'K';

	} elsif ( $size < (1024 ** 3) ) {
		$size = $size / ( 1024 ** 2 );
		$size = $size < 10 ? sprintf( "%.1f", $size ) : int( $size );
		return $size . 'M';

	} elsif ( $size < (1024 ** 4) ) {
		$size = $size / ( 1024 ** 3 );
		$size = $size < 10 ? sprintf( "%.1f", $size ) : int( $size );
		return $size . 'G';

	} else {
		return 'Huge!';

	}
}

# Convert binary to base64
# Arguments are string references
# We can't alter the original binary scalar, and we can't do
# a copy -> buffer -> new because that would involve duplicating
# the memory, chewing more, and we might be converting large
# binary files, so we need to substr sections out
sub BinToBase64 {
	my $class = shift;
	my $binary = isa( $_[0], 'SCALAR' ) ? shift : return undef;
	my $base64 = isa( $_[0], 'SCALAR' ) ? shift : return undef;
	unless ( length $$binary ) {
		$$base64 = $$binary;
		return 1;
	}

	# Since we need to call a function directly
	# load the Base64 module directly
	Class::Autouse->load( 'MIME::Base64' );

	# Encode the binary data in blocks
	my $offset = 0;
	my $buffer = '';
	while ( $buffer = substr( $$binary, $offset, $base64_encode_buffer_size ) ) {
		# Encode, and add the block to the output string
		$$base64 .= MIME::Base64::encode( $buffer );
		$offset += $base64_encode_buffer_size;
	}

	return 1;
}

# Same as BinToBase64 but in the opposite direction
# Unlike the other function however, we don't need
# the source when we are done with it
sub Base64ToBin {
	my $class = shift;
	my $base64 = isa( $_[0], 'SCALAR' ) ? shift : return undef;
	my $binary = isa( $_[0], 'SCALAR' ) ? shift : return undef;
	unless ( length $$base64 ) {
		$$binary = $$base64;
		return 1;
	}

	# Load the Base64 module
	Class::Autouse->load( 'MIME::Base64' );

	# Preparse the input
	$$base64 =~ s/\s//g;

	# Decode the Base64 data in blocks
	my $offset = 0;
	my $buffer = '';
	while( $buffer = substr( $$base64, $offset, $base64_decode_buffer_size ) ) {
		# Decode and add the block to the output string
		$$binary .= MIME::Base64::decode( $buffer );
		$offset += $base64_decode_buffer_size;
	}

	return 1;
}

# Determines the length of the binary data of a decoded
# base64 scalar based only on the starting size
sub Base64Length {
	my $class = shift;
	my $encoded = shift;
	my $full_lines = int( $encoded / 77 );
	my $end_bytes = $encoded % 77;
	my $size = ( $full_lines * 76 * 3 / 4 )
		+ ( ( $end_bytes - 1 ) * 3 / 4 );
	return $size;
}

# Generate a random string of a certain length
sub random {
	my $class = shift;
	my $size = shift;
	$size = '16' unless $size;

	# Create the character array
	my @chars = ( ('A'..'Z'), ('0'..'9') );
	my $chars_length = scalar @chars;
	return join '',
		map { $chars[$_] }
		map { int rand $chars_length }
		( 1 .. $size );
}

1;
