package CVSMonitor::Graph;

# The CVSMonitor::Graph package provides an abstract class for all Graph classes
# to inherit from. It provides common methods, and tools for graphing.

use strict;
use UNIVERSAL 'isa';
use Class::Autouse 'Class::Inspector';





#####################################################################
# Standard Access Methods

# All graphs are required to implement a certain set of features. 
# The access methods below implement these.

# This exists just to feed calls off to sub classes.
sub new {
	my $class = shift;
	my $type = shift;
	return undef unless Class::Inspector->installed( "CVSMonitor::Graph::$type" );
	"CVSMonitor::Graph::$type"->new( @_ );
}

# Initialise the standard properties
sub _init {
	my $self = shift;
	$self->{title}      = undef;
	$self->{width}      = 400;
	$self->{height}     = 300;
	$self->{show_start} = 1;
	1;
}

# Get and set the title of the graph
sub getTitle { $_[0]->{title} || $_[0]->defaultTitle }
sub setTitle { $_[0]->{title} = $_[1] }

# Provide a default title
sub defaultTitle { "A Graph" }

# Get and set the height and width
sub getHeight { $_[0]->{height} }
sub getWidth  { $_[0]->{width} }
sub setHeight { $_[0]->{height} = $_[1] }
sub setWidth  { $_[0]->{width} = $_[1] }

# Get and set the show_start flag
sub getShowStart { $_[0]->{show_start} }
sub setShowStart { $_[0]->{show_start} = $_[1] }





#####################################################################
# Image Generation Methods

# The Graphs will generally be produced in gif or png formats.
# The following methods help decide which to use.

# Get the image type of the resulting image.
# This will be dependant on the version of GD.
sub imageType {
	my $class = shift;

	# Load GD if it isn't already
	Class::Autouse->load( 'GD' ) or return undef;

	# Find out it's version
	my $version = $GD::VERSION;
	$version > 1.19 ? 'png' : 'gif';
}

# Get the content type of the image
sub contentType {
	my $class = shift;
	my $type = $class->imageType or return undef;
	"image/$type";
}





#####################################################################
# Generation convenience methods

# Same as the ->gd method, but returns the image data directly.
# The type of image can be determined from the imageType method.
sub image {
	my $self = shift;

	# Get the graph
	my $gd = $self->gd() or return undef;

	# Convert to an image and return
	my $type = $self->imageType or return undef;
	$gd->$type();
}

# Display to the user.
# For now just pass through to sendToBrowser.
sub display { shift->sendToBrowser(@_) }

# Send the graph immediately to the user's browser ( if this is a CGI ).
# Returns true if sent successfully, undef otherwise.
sub sendToBrowser {
	my $self = shift;

	# Load the CGI module
	Class::Autouse->load( 'CGI' ) or return undef;

	# Get the generated image
	my $image = $self->image() or return undef;

	# Get the content type and generate the CGI header
	my $type = $self->contentType or return undef;
	print STDOUT CGI::header( -type => $type );
	print STDOUT $image;

	1;
}






#####################################################################
# Slice Math Methods

# Graphing modules often need to do a range of math on "slices", basically
# an array reference containing numbers. The slice methods below provide that
# functionality.

# Take two equals length array refs and add their contents
sub addSlices {
	my $class = shift;
	my $first = isa( $_[0], 'ARRAY' ) ? shift : return undef;
	my $second = isa( $_[0], 'ARRAY' ) ? shift : return undef;;
	return undef unless scalar @$first == scalar @$second;

	# Create the new array
	my @added = ();
	foreach ( 0 .. (scalar @$first - 1) ) {
		push @added, $first->[$_] + $second->[$_];
	}

	\@added;
}

# Make all the values in a slice the negative
sub negateSlice {
	my $class = shift;
	my $slice = isa( $_[0], 'ARRAY' ) ? shift : return undef;

	# Negate and return the slice as a convenience
	$_ = -$_ foreach @$slice;
	$slice;
}

# Make a slice accumulator
sub accumulateSlice {
	my $class = shift;
	my $slice = isa( $_[0], 'ARRAY' ) ? shift : return undef;

	# Build the accumlated totals
	my @accum = ();
	my $total = 0;
	foreach ( @$slice ) {
		$total += $_;
		push @accum, $total;
	}

	\@accum;
}

# Get the minimum for one or more slices
sub sliceMin {
	my $class = shift;
	my @slices = @_;
	return undef if scalar grep { ! isa( $_, 'ARRAY' ) } @slices;

	# Find the minimum
	my $min = $slices[0]->[0];
	foreach ( map { @$_ } @slices ) {
		$min = $_ if $_ < $min;
	}

	$min;
}

# Get the minimum for one or more slices
sub sliceMax {
	my $class = shift;
	my @slices = @_;
	return undef if scalar grep { ! isa( $_, 'ARRAY' ) } @slices;

	# Find the minimum
	my $max = $slices[0]->[0];
	foreach ( map { @$_ } @slices ) {
		$max = $_ if $_ > $max;
	}

	$max;
}

# Divide second slice into the first slice into another, good for getting ratios.
# Will return 0 on a divide-by-zero issue.
sub divideSlices {
	my $class = shift;
	my $first = isa( $_[0], 'ARRAY' ) ? shift : return undef;
	my $second = isa( $_[0], 'ARRAY' ) ? shift : return undef;;
	return undef unless scalar @$first == scalar @$second;

	# Create the new array
	my @divided = ();
	foreach ( 0 .. (scalar @$first - 1) ) {
		if ( $second->[$_] == 0 ) {
			push @divided, 0;
		} else {
			push @divided, $first->[$_] / $second->[$_];
		}
	}

	\@divided;
}

1;

__END__

# Copyright (C) 2002-2004 Adam Kennedy
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Should you wish to utilise this software under a different licence, 
# please contact the author.
